@@ -3,6 +3,7 @@ use feature ':5.10';
 use strict;
 use warnings;
 
+my $ModulePath = __PACKAGE__->path;
 my $GetRetried = __PACKAGE__->retry;
 my $ClassOrder = [
     [qw|MailboxFull MesgTooBig ExceedLimit Suspend HasMoved NoRelaying UserUnknown
@@ -18,6 +19,7 @@ my $ClassOrder = [
         NotAccept MailerError NoRelaying SyntaxError OnHold
     |],
 ];
+
 sub retry {
     # Reason list better to retry detecting an error reason
     # @return   [Array] Reason list
@@ -38,6 +40,17 @@ sub index {
     |];
 }
 
+sub path {
+    # Returns Sisimai::Reason::* module path table
+    # @return   [Hash] Module path table
+    # @since    v4.25.6
+    my $class = shift;
+    my $index = __PACKAGE__->index;
+    my $table = {};
+    $table->{ __PACKAGE__.'::'.$_ } = 'Sisimai/Reason/'.$_.'.pm' for @$index;
+    return $table;
+}
+
 sub get {
     # Detect the bounce reason
     # @param    [Sisimai::Data] argvs   Parsed email object
@@ -62,8 +75,7 @@ sub get {
             # Check the value of Diagnostic-Code: and the value of Status:, it is a
             # deliverystats, with true() method in each Sisimai::Reason::* class.
             my $p = 'Sisimai::Reason::'.$e;
-            (my $modulepath = $p) =~ s|::|/|g;
-            require $modulepath.'.pm';
+            require $ModulePath->{ $p };
 
             next unless $p->true($argvs);
             $reasontext = $p->text;
@@ -114,8 +126,7 @@ sub anotherone {
         for my $e ( @{ $ClassOrder->[1] } ) {
             # Trying to match with other patterns in Sisimai::Reason::* classes
             my $p = 'Sisimai::Reason::'.$e;
-            (my $modulepath = $p) =~ s|::|/|g;
-            require $modulepath.'.pm';
+            require $ModulePath->{ $p };
 
             next unless $p->match($diagnostic);
             $reasontext = lc $e;
@@ -175,8 +186,7 @@ sub match {
         # Check the value of Diagnostic-Code: and the value of Status:, it is a
         # deliverystats, with true() method in each Sisimai::Reason::* class.
         my $p = 'Sisimai::Reason::'.$e;
-        (my $modulepath = $p) =~ s|::|/|g;
-        require $modulepath.'.pm';
+        require $ModulePath->{ $p };
 
         next unless $p->match($diagnostic);
         $reasontext = $p->text;
@@ -185,12 +195,10 @@ sub match {
     return $reasontext if $reasontext;
 
     # Check the value of $typestring
-    my $typestring = '';
-       $typestring = uc($1) if uc($argv1) =~ /\A(SMTP|X-.+);/;
+    my $typestring = uc($argv1) =~ /\A(SMTP|X-.+);/ ? uc($1) : '';
     if( $typestring eq 'X-UNIX' ) {
         # X-Unix; ...
         $reasontext = 'mailererror';
-
     } else {
         # Detect the bounce reason from "Status:" code
         require Sisimai::SMTP::Status;
@@ -537,7 +545,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2019 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2020 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
